% ALPAO SDK Wrapper
%
% This class help to drive Alpao deformable mirrors
%
classdef asdkDM < handle
    properties
        nAct = 0;
    end
    
    properties %(SetAccess = private, Hidden=true )
        dmId = -1;
    end
    
    methods
        function obj = asdkDM( serialName )
            %% Default class constructor
            
            obj.dmId = AOcontrol.DMdriver.asdkMex( 0, serialName ); %added AOcontrol.DMdriver (Alfred)
            obj.nAct = AOcontrol.DMdriver.asdkMex( 5, obj.dmId, 'NbOfActuator' );
        end

        function delete( obj )
            %% Default class destructor
            
            AOcontrol.DMdriver.asdkMex( 1, obj.dmId );
        end

        function Send( obj, values )
            %% Send values to the mirror
            % Parameters:
            %   values - vector of nAct values to send.  
            if ( numel( values ) ~=obj.nAct )
                error( 'ASDK:DM:InvalidSize', 'Invalid number of element.');
            end
            AOcontrol.DMdriver.asdkMex( 2, obj.dmId, values );
        end

        function Reset( obj )
            %% Flat the mirror
            
            AOcontrol.DMdriver.asdkMex( 7, obj.dmId );
        end
        
        function SendPattern( obj, values, nRepeat )
            %% Send multiple values to the mirror and repeat it
            % Parameters:
            %   values  - vector of nAct values to send.
            %   nRepeat - number of time to send the vector ( '0' mean
            %             infinitly )
            
            AOcontrol.DMdriver.asdkMex( 3, obj.dmId, values, numel( values ) / obj.nAct, nRepeat );
        end
        
        function Stop( obj )
            %% Stop current transfer to mirror
            
            AOcontrol.DMdriver.asdkMex( 4, obj.dmId );
        end
        
        function value = Get( obj, cmdName )
            %% Get value from SDK
            % Parameters:
            %   cmdName - name of the value.  
            
            value = AOcontrol.DMdriver.asdkMex( 5, obj.dmId, cmdName );
        end
        
        function Set( obj, cmdName, value )
            %% Get value from SDK
            % Parameters:
            %   cmdName - name of the value. 
            %   value   - value to set (string or numerical)
            
            AOcontrol.DMdriver.asdkMex( 6, obj.dmId, cmdName, value );
        end
    end
        
    methods (Hidden)
        function SendPatternSingle( obj, values, nRepeat, actuatorId )
            %% Send multiple values to the mirror and repeat it
            % Parameters:
            %   values  - vector of nAct values to send.
            %   nRepeat - number of time to send the vector ( '0' mean
            %             infinitly )
            %   actuId  - actuator number in [1:nAct]
            
            AOcontrol.DMdriver.asdkMex( 8, obj.dmId, values, actuatorId, nRepeat );
        end
    end
    
    methods (Static)
        function PrintLastLog()
            AOcontrol.DMdriver.asdkMex( -1 );
        end
        function [message, id] = GetLastLog()
            [message, id] = AOcontrol.DMdriver.asdkMex( -2 );
        end
    end
end