classdef SHctrl<handle
    %AOctrl is in charge of communicating with AO hardwares (SH sensor camera, Deformable mirror)
    %and the storage of wavefront measurment data, for separate ROIs during
    %imaging and scanning.
    
    properties
        %handles
        hAOROIctrl
        
        %internal properties
        SHdata %temporary storage of SHdata.
        currentROIid %roiidloaded from AOROIctrl
        DMdatatoload % temporary storage of DM data to be loaded
        DMdata
        %SH sensor (Andor camera) properties
        cooleron
        settemp
        emgain
        expotime
        XPixels
        YPixels
        %DM control properties
        
        istakingdataSH
        
        initializedSH
        
        
    end
    %lifecycle
    methods
        
        function obj=SHctrl(AOROIctrl)
        obj.DMdata=[];
        obj.hAOROIctrl = AOROIctrl;
        obj.initializedSH=0;
        
        
        
        end
       
        function delete(obj)
            
            
            if obj.initializedSH==true
                obj.deinitializeSH()
            end
            
        end
    end
    %user methods
    methods
  
        
        function SHdata=takeSHdata(obj)
            fnSHdata='fnSHdata';
            [ret]=SetExposureTime(obj.expotime);%use current exposure time.
            CheckWarning(ret);
            [ret]=SetShutter(1,1, 27, 27); %open shutter
            CheckWarning(ret);
            [ret] = StartAcquisition();                   
            CheckWarning(ret);
            %fprintf('takeSHdata')
            [ret] = WaitForAcquisition();
            CheckWarning(ret);
%             [ret,imgdata]=GetMostRecentImage16(XPixals*YPixals);
%             CheckWarning(ret);
            [ret]=AbortAcquisition;
            CheckWarning(ret);
            [ret]=SetShutter(1,2, 27, 27); %close shutter
            CheckWarning(ret);
            [ret, imageData] = GetMostRecentImage16(obj.XPixels*obj.YPixels);
            CheckWarning(ret);
            SHdata=flipdim(transpose(reshape(imageData, obj.XPixels,obj.YPixels)),1);
        end
        
        function Aqstart(obj)
           %function called when start grabing, when AO is enabled.
        end
        
        function Aqstop(obj)
        %function called when stop grabing, when AO is enabled.
        end
        
    end
    
    %Camera setting methods
    methods
        function success=turncooleron(obj)
            [ret]=CoolerON();
            if isequal(ret,atmcd.DRV_SUCCESS)
                success=true;
                obj.cooleron=true;
            else
                success=false;
               CheckWarning(ret)
            end
        end
        
        function success=turncooleroff(obj)
            [ret]=CoolerOFF();
            if isequal(ret,atmcd.DRV_SUCCESS)
                success=true;
            obj.cooleron=false;
            else
                success=false;
             CheckWarning(ret)
            end
        end
        
        function success=setcoolertemp(obj,temp)
            [ret]=SetTemperature(temp);
            if isequal(ret,atmcd.DRV_SUCCESS)
                success=true;
            obj.settemp=temp;
            else
                success=false;
             CheckWarning(ret)
            end
           
             
        end
        
        function temp=checktemp(obj)
            [ret,temp]=GetTemperature();
            
            %CheckWarning(ret); %commented becaused always return
            %temperature not stablized.
        end
        
        function success=setemgain(obj,gain)
            
            [ret]=SetEMCCDGain(gain);
            if isequal(ret,atmcd.DRV_SUCCESS)
                success=true;
            obj.emgain=gain;
            else
                success=false;
             CheckWarning(ret);
             disp(ret)
            end
        
        end
        
    end
    %methods for initializing and deinitializing
    methods 
        
        
        function success=initializeSH(obj)
            %fprintf('initializeSH')
            %success=1;
            if obj.initializedSH==1
               msgbox('already initialized','Error')
               success=0;
            else
                ret=AndorInitialize('');
                if isequal(ret,atmcd.DRV_SUCCESS)
                success=true;
                obj.initializedSH=1;
                %initialized, then set all default camera settings.
                obj.cooleron=0;
                obj.expotime=0.01;
                obj.emgain=2;
                obj.settemp=15;
                [ret]=SetAcquisitionMode(1);     %   Set acquisition mode; 5 for RTA;1 for single scan
                CheckWarning(ret);
                [ret]=SetExposureTime(0.02);    %   Set exposure time in second
                CheckWarning(ret);
                [ret]=SetReadMode(4);        %   Set read mode; 4 for Image
                CheckWarning(ret);
                [ret]=SetTriggerMode(0); %use internal trigger mode.
                CheckWarning(ret);
                [ret,XPixels, YPixels]=GetDetector;           %   Get the CCD size
                CheckWarning(ret);
                [ret]=SetImage(1, 1, 1, XPixels, 1, YPixels); %   Set the image size
                CheckWarning(ret);
%                 [ret] = SetEMGainMode(3); %EM gain mode 3 for Real EM gain
%                 CheckWarning(ret);
%                 [ret]= SetEMCCDGain(obj.emgain); % set default em gain 2;
%                 CheckWarning(ret);
                [ret] = SetFanMode(0); % fan mode, 1 for low.
                CheckWarning(ret);
%                 [ret]=SetVerticalSpeed();
%                 CheckWarning(ret);
                [ret,XPixels, YPixels]=GetDetector;           %   Get the CCD size
                CheckWarning(ret);
                obj.XPixels=XPixels;
                obj.YPixels=YPixels;
                [ret]=SetImage(1, 1, 1, XPixels, 1, YPixels); %   Set the image size Full now 
                %SYNOPSIS : [ret] = SetImage(hbin, vbin, hstart, hend, vstart, vend)
                %
                % INPUT hbin: number of pixels to bin horizontally.
                %       vbin: number of pixels to bin vertically.
                %       hstart: Start column (inclusive).
                %       hend: End column (inclusive).
                %       vstart: Start row (inclusive).
                %       vend: End row (inclusive).
                CheckWarning(ret);
                [ret]=SetShutter(1,1, 27, 27); 
                CheckWarning(ret);
                [ret]=SetVSSpeed(1); % set vertial shift speed using index, 1 for 1.13
                CheckWarning(ret);
                [ret]=SetHSSpeed(0,3); %Set horizontal shift speed (read out speed) 0-3 int. 30, 20, 10, 1 resp.
                CheckWarning(ret);
                else
                success=0;
                CheckWarning(ret);
                disp(ret)
                end
            end
%                
            
        end
%         
        
        
        function success=deinitializeSH(obj)
            %fprintf('deinitializeSH')
            safetemp=0; % set the minimum temperature when deinitializing.\
            
            if checktemp(obj)<safetemp
                msgbox('Sensor Temperature too low, set to 15 and wait','Waring')
                success=0;
            else
                [ret]=AndorShutDown;
                if isequal(ret,atmcd.DRV_SUCCESS)
                success=1;
                obj.initializedSH=0;
                else
                success=0;
                CheckWarning(ret);
                disp(ret)
                end
            
            
            end
        end
    
    
    
        function good=iscameraready(obj)
            %check if camera is ready for recording, in addition to being
            %initialized 
            if obj.initializedSH==0
                good=false;
                return
            end
            %additional thing to check goea here.
            
            good=true;
        end
    end
end
                
                