% Rui changes some paras
% Calibration example. Adjust the parameters of your lenslet array and use
% your SHWFS reference image.


%% parameters
shstruct = struct();
shstruct.calibration_date = datestr(now, 'yyyymmddHHMMSS');
shstruct.use_bg = 0;               % subtract background from images
shstruct.use_gausscorrection = 1; %using gauss correction for DM calibration
%shstruct.thresh_binary_img = 0.06;%0.08 % threshold for binary image coarse grid
shstruct.thresh_fine_img = 0.55;  % threshold for correlation image area detect
shstruct.npixsmall = 8; %8           % remove objects less than npixsmall
shstruct.npixsmall_corr = 6;         % remove objects less than npixsmall for corr-image
shstruct.strel_rad = 8;              % strel ratio (image processing in coarse grid)
shstruct.strel_rad_corr = 8;         % strel ratio (image processing in corr-image)
shstruct.coarse_grid_radius = round((500/13)/2)-1;  % radius for coarse grid
shstruct.percent = 0.8;%0.2            % threshold used in centroid fine grid
                                       % this value shoule be bigger when
                                       % more noisy of the image
                                       % scale estimated radius in the fine grid 
% define the size and sigma of gaussian filter used for psf
shstruct.psfsize_DM = 31;
shstruct.psfsize_sample = 31;
shstruct.psfsigma_sample = 1;
shstruct.psfsigma_DM = 1;
% this caused the fine grid to be smaller than the course grid
shstruct.multiply_est_radius = 0.9; %1/sqrt(2)
shstruct.centroid = @centroid;     % centroid algorithm
% lenslet documentation
shstruct.totlenses = 170;
shstruct.flength = 25.6e-3;
shstruct.pitch = 500e-6;
% camera documentation
shstruct.camera_pixsize = 13*1e-6;
shstruct.maxinteger = 2^16 - 1;       % maximum integer value for the image
% params Dai modal wavefront estimation
shstruct.sa_radius_m = shstruct.pitch/2;
shstruct.dai_n_zernike = 10;%5  % max radial order of the Zernike polynomials


%% pre-process of sh_flat
% replace your reference SHWFS images here
sh_flat=imread('sample_ref.tif');
sh_flat_bg=imread('sample_bg.tif');
% subtract background
sh_flat = int16(sh_flat)-int16(sh_flat_bg); 
% aperture template cut 
imgtemp = int16(zeros(1024,1024));
xcenter = 505;
ycenter = 505;
imgrad = 290;%real size is 280
for x = 1:1024
    for y = 1:1024
        if sqrt((x-xcenter)^2+(y-ycenter)^2)<=imgrad
            imgtemp(x,y)=1;
        else
            imgtemp(x,y)=0;
        end
    end
end
sh_flat = sh_flat.*imgtemp;
%Gauss correction
gautemp = fspecial('gaussian',1024,260);% the radius of the gauss temp can be smaller 
gautemp =gautemp.^3; % 3 power is better than 2 power
gautemp = (gautemp-min(min(gautemp)))./(max(max(gautemp))-min(min(gautemp)));
if shstruct.use_gausscorrection
    sh_flat = int16(double(sh_flat)./gautemp);
else
    sh_flat;
end

% generate threshould for course grid cal
shstruct.thresh_course_img = graythresh(sh_flat);%%%this is crucial for grid generation

%% run the calibration
%shwfs_calibrate;

mkdir(sprintf('calibration-%s', shstruct.calibration_date));
%% background
% sfigure(1);
% imshow(sh_flat_bg);
% title('background');
% drawnow();
% pause(.1);

%% flat mirror reference
% Rui<this section do nothing>
figure(2);
imshow(sh_flat);
imagesc(sh_flat);
title('reference image with gauss correction');
iimin = min(sh_flat(:));
iimax = max(sh_flat(:));
fprintf('[min, max] = [%.2f %.2f]\n', iimin, iimax);

shstruct.sh_flat = sh_flat;
shstruct.sh_flat_bg = sh_flat_bg;

if exist('scflat', 'var')
    shstruct.scflat = scflat;
end

% filename_blank = ...
%     sprintf('calibration-%s/shstruct.blank.mat', ...
%     shstruct.calibration_date);
% ask_confirm('continue?');
% 
% save(filename_blank, 'shstruct');
% fprintf('saved %s\n', filename_blank);

%% coarse grid with image processing
%load(filename_blank);
shstruct = shwfs_make_coarse_grid(shstruct);
filename_coarse = ...
    sprintf('calibration-%s/shstruct.coarse.mat', ...
    shstruct.calibration_date);
%ask_confirm('continue?');

save(filename_coarse, 'shstruct');
fprintf('saved %s\n', filename_coarse);
%% finer grid
load(filename_coarse);
shstruct = shwfs_make_fine_grid(shstruct);
filename_fine = ...
    sprintf('calibration-%s/shstruct.fine.mat', ...
    shstruct.calibration_date);
save(filename_fine, 'shstruct');
fprintf('saved %s\n', filename_fine);



%% get slope 

nspots = shstruct.nspots;
ref_cernter = shstruct.ord_centres;

img = SHdata(:,:,i);
img = img - int16(sh_flat_bg); 
aberrated = shwfs_get_centres(img, shstruct, shstruct.centroid);
slope(:,i) = reshape(aberrated,nspots*2,1) - reshape(ref_cernter,nspots*2,1);
     

end
Geninv = pinv(slope);


