% SHWFS_GET_CENTRES gets the spots centres.
%   CENTRES = SH_GET_CENTRES(IMG, SHSTRUCT, MYFUNCENTROID) gets the pixel
%   coordinates of the spots from an image in doubles of the Shack-Hartmann
%   wavefront sensor. centres is stored in the (x, y) plot() reference
%   frame after imshow(). The origin is in the top left corner.
%   centres(:, 1) are the displacements in X (left to right).
%   centres(:, 2) are the displacements in Y (top to bottom).
%   Displacements are ordered according to shstruct.enumeration.
%   centres(i, 1) corresponds to the i-th spot (shstruct.enumeration(i)).
%
% Author: Jacopo Antonello, <jack@antonello.org>
% Technische Universiteit Delft

function centres = shwfs_get_centres(img, shstruct, ...
    myfuncentroid)

nspots = shstruct.nspots;
ordgrid = shstruct.ord_sqgrid;
centres = zeros(nspots, 2);
thresh_fine_img = shstruct.thresh_fine_img;
%npixsmall_corr = shstruct.npixsmall_corr;
%strel_rad_corr = shstruct.strel_rad_corr;
psf_size = shstruct.psfsize_sample;
psf_sigma = shstruct.psfsigma_sample;

img_corr = corsscorrelation(psf_size,psf_sigma,img);


for ith = 1:nspots
    cc = ordgrid(ith, :);
    subimage = img(cc(3):cc(4), cc(1):cc(2));
    subimage_corr = img_corr(cc(3):cc(4), cc(1):cc(2));
    
    iimax = max(max(subimage_corr));
    iimin = min(min(subimage_corr));
    
    level = (iimax - iimin)*thresh_fine_img; 
    subimage_corr = subimage_corr - iimin;
    subimage_corr_bw = im2bw(subimage_corr, level);
    
    % find the max area (20170224)
    subimage_corr_bw_label = bwlabel(subimage_corr_bw);
    stats = regionprops(subimage_corr_bw);
    area = cat(1,stats.Area);
    index = find(area == max(area));
    subimage_corr_bw = ismember(subimage_corr_bw_label,index);
    
%     
%     %remove small objects
%     subimage_corr_bw = bwareaopen(subimage_corr_bw, npixsmall_corr);
%     % remove edges
%     se = strel('disk',strel_rad_corr);
%     subimage_corr_bw = imclose(subimage_corr_bw, se);
 
    if sum(sum(subimage_corr_bw))==0
        subimage_corr_bw(round(size(subimage_corr_bw,1)/2),...
            round(size(subimage_corr_bw,1)/2)) = 1;
    end
        
    
    subimage_thr = subimage.*int16(subimage_corr_bw);
    
    % level = (iimax - iimin)*shstruct.percent + iimin;   
     dd = myfuncentroid(subimage_thr);
     
    % centroid returns in column major order
    centres(ith, :) = [cc(1)+dd(2)-1, cc(3)+dd(1)-1];
end


    
    % reduce scattering pattern
%     h_freq_subimage=highpassImage(subimage);
%     iimin = min(min(h_freq_subimage));
%     iimax = max(max(h_freq_subimage));
%     level = (iimax - iimin)*shstruct.percent + iimin;
%     dd = myfuncentroid(h_freq_subimage, level);  
    
%     iimin = min(min(subimage));
%     iimax = max(max(subimage));
%     level = (iimax - iimin)*shstruct.percent + iimin;
%     dd = myfuncentroid(subimage, level);
    

% 
%% draw the correlation result
figure;
imshow(img_corr);
hold on; 
grid_centres = shstruct.ord_centres;
for i=1:nspots
    cc = ordgrid(i, :);
    rectangle('Position', ...
        [cc(1), cc(3), cc(2)-cc(1)+1, cc(4)-cc(3)+1], ...
        'LineWidth', 1, 'EdgeColor', 'r');
    plot(grid_centres(i,1),grid_centres(i,2),'+y');
      
    text(cc(1), cc(3), ...
        sprintf('%d', i), 'Color', 'g');    
end
hold off;
%% draw the grid on each image 

figure;
imshow(img);
imagesc(img);
hold on; 
grid_centres = shstruct.ord_centres;
centres_deltas = centres - grid_centres;
for i=1:nspots
    cc = ordgrid(i, :);
    rectangle('Position', ...
        [cc(1), cc(3), cc(2)-cc(1)+1, cc(4)-cc(3)+1], ...
        'LineWidth', 1, 'EdgeColor', 'r');   
    plot(centres(i,1),centres(i,2),'xr');
    plot(grid_centres(i,1),grid_centres(i,2),'xy');
    quiver(grid_centres(i,1),grid_centres(i,2),centres_deltas(i,1),centres_deltas(i,2))
    
    text(cc(1), cc(3), ...
        sprintf('%d', i), 'Color', 'g');    
end
hold off;
end