
function centres = shwfs_get_centres_modified(img, shstruct, ...
    myfuncentroid)

nspots = shstruct.nspots;
ordgrid = shstruct.ord_sqgrid;
centres = zeros(nspots, 3);
thresh_fine_img = shstruct.thresh_fine_img;
%npixsmall_corr = shstruct.npixsmall_corr;
%strel_rad_corr = shstruct.strel_rad_corr;
psf_size = shstruct.psfsize_sample;
psf_sigma = shstruct.psfsigma_sample;

img_corr = corsscorrelation(psf_size,psf_sigma,img);

for ith = 1:nspots
    cc = ordgrid(ith, :);
    subimage = img(cc(3):cc(4), cc(1):cc(2));
    subimage_corr = img_corr(cc(3):cc(4), cc(1):cc(2));    
    iimax = max(max(subimage_corr));
    iimin = min(min(subimage_corr));
    
    if iimax < 0.2 %firstly make sure it's not a bad spot
        
        centres(ith, 1:2) = [round((cc(1)+cc(2))/2), round((cc(3)+cc(4))/2)];
        centres(ith, 3) = 1;
        
    else
        level = (iimax - iimin)*thresh_fine_img;
        subimage_corr = subimage_corr - iimin;
        subimage_corr_bw = im2bw(subimage_corr, level);
        
        % find the max area
        subimage_corr_bw_label = bwlabel(subimage_corr_bw);
        stats = regionprops(subimage_corr_bw);
        area = cat(1,stats.Area);
        index = find(area == max(area));
        subimage_corr_bw = ismember(subimage_corr_bw_label,index);
        
        if sum(sum(subimage_corr_bw))==0
            subimage_corr_bw(round(size(subimage_corr_bw,1)/2),...
                round(size(subimage_corr_bw,1)/2)) = 1;
        end
     
        subimage_thr = subimage.*int16(subimage_corr_bw);
        
        % level = (iimax - iimin)*shstruct.percent + iimin;
        dd = myfuncentroid(subimage_thr);        
        % centroid returns in column major order
        centres(ith, 1:2) = [cc(1)+dd(2)-1, cc(3)+dd(1)-1];       
        
        % for signal to background ratio calculation
        bkground_corr_bw = ~subimage_corr_bw;
        subimage_bkground = subimage.*int16(bkground_corr_bw);
        signal_pN = sum(sum(subimage_corr_bw));
        bkground_pN = sum(sum(bkground_corr_bw));
        subimage_sbr = (sum(sum(subimage_thr))/signal_pN)/(sum(sum(subimage_bkground))/bkground_pN);
        centres(ith, 3) = subimage_sbr;
        
    end 
        
end


% draw the slope on each wavefront image

figure;
imshow(img);
imagesc(img);
hold on; 
grid_centres = shstruct.ord_centres;
centres_deltas = centres(:,1:2) - grid_centres;
for i=1:nspots
    cc = ordgrid(i, :);
    rectangle('Position', ...
        [cc(1), cc(3), cc(2)-cc(1)+1, cc(4)-cc(3)+1], ...
        'LineWidth', 1, 'EdgeColor', 'r');   
    plot(centres(i,1),centres(i,2),'xr');
    plot(grid_centres(i,1),grid_centres(i,2),'xy');
    quiver(grid_centres(i,1),grid_centres(i,2),centres_deltas(i,1),centres_deltas(i,2))
    
    text(cc(1), cc(3), ...
        sprintf('%d', i), 'Color', 'g'); 
    text(cc(1), cc(4), ...
        sprintf('%f',centres(i,3)), 'Color', 'g');     
end
hold off;


end